<?php

declare(strict_types=1);

namespace app\vendor\apexl\io\src\Io\interfaces;

use apexl\Io\interfaces\AbstractHook;

interface ProvidesHooksInterface
{
    /**
     * @return class-string<AbstractHook>[]
     */
    public function hooks(): array;
}
