<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

use apexl\Vault\interfaces\driver;

interface LoaderConditionInterface
{
    public function applyToQuery(driver $query): driver;

    public function shouldWrap(): bool;
}
