<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

/** @Todo rename to HookInterface */
interface AbstractHook
{
    /**
     * @return string[]
     */
    public function hooks(): array;
}
